@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteTypes.mli 9911 2009-06-19 10:16:23Z sacerdot $ *)
+(* $Id: grafiteTypes.mli 12484 2013-02-02 00:38:05Z sacerdot $ *)
 
 exception Option_error of string * string
 exception Statement_error of string
@@ -31,48 +31,21 @@ exception Command_error of string
 
 val command_error: string -> 'a   (** @raise Command_error *)
 
-type incomplete_proof = {
-  proof: ProofEngineTypes.proof;
-  stack: Continuationals.Stack.t;
-}
-
-type proof_status =
-    No_proof
-  | Incomplete_proof of incomplete_proof
-  | Proof of ProofEngineTypes.proof
-  | Intermediate of Cic.metasenv
-
-class status :
+class virtual status :
  string ->
   object ('self)
-   method moo_content_rev: GrafiteMarshal.moo
-   method set_moo_content_rev: GrafiteMarshal.moo -> 'self
-   method proof_status: proof_status
-   method set_proof_status: proof_status -> 'self
-   method objects: UriManager.uri list
-   method set_objects: UriManager.uri list -> 'self
-   method coercions: CoercDb.coerc_db
-   method set_coercions: CoercDb.coerc_db -> 'self
-   method automation_cache:AutomationCache.cache
-   method set_automation_cache:AutomationCache.cache -> 'self  
+   (* Warning: #stack and #obj are meaningful iff #ng_mode is `ProofMode *)
+   inherit NTacStatus.tac_status
+   inherit NCicLibrary.dumpable_status
+   inherit NCicLibrary.status
+   inherit NCicExtraction.status
+   inherit OcamlExtractionTable.status
+   inherit GrafiteParser.status
+   inherit TermContentPres.status
    method baseuri: string
    method set_baseuri: string -> 'self
    method ng_mode: [`ProofMode | `CommandMode]
    method set_ng_mode: [`ProofMode | `CommandMode] -> 'self
-   (* Warning: #stack and #obj are meaningful iff #ng_mode is `ProofMode *)
-   inherit NTacStatus.tac_status
   end
 
-val dump_status : status -> unit
-
-  (** list is not reversed, head command will be the first emitted *)
-val add_moo_content: GrafiteMarshal.ast_command list -> status -> status
-
-val get_current_proof: status -> ProofEngineTypes.proof
-val get_proof_metasenv: status ->  Cic.metasenv
-val get_stack: status -> Continuationals.Stack.t
-val get_proof_context : status -> int -> Cic.context
-val get_proof_conclusion : status -> int -> Cic.term
-
-val set_stack: Continuationals.Stack.t -> status -> status
-val set_metasenv: Cic.metasenv -> status -> status
+module Serializer: NCicLibrary.SerializerType with type dumpable_status = status
