@@ -23,13 +23,12 @@
  * http://helm.cs.unibo.it/
  *)
 
-val set_reference_of_oxuri: (UriManager.uri -> NReference.reference) -> unit
-
 module Matcher32:
 sig
   (** @param l3_patterns level 3 (CIC) patterns (AKA cic_appl_pattern) *)
   val compiler :
-    (CicNotationPt.cic_appl_pattern * int) list ->
+   #NCic.status ->
+    (NotationPt.cic_appl_pattern * int) list ->
       (NCic.term ->
         ((string * NCic.term) list * NCic.term list * int) option)
 end
