@@ -11,26 +11,26 @@
 
 (* $Id: nCic.ml 9058 2008-10-13 17:42:30Z tassi $ *)
 
-val set_reference_of_oxuri: (UriManager.uri -> NReference.reference) -> unit
+val debug : bool ref
 
 val disambiguate_term :
+  #NCicCoercion.status ->
   context:NCic.context ->
   metasenv:NCic.metasenv -> 
   subst:NCic.substitution ->
-  expty:NCic.term option ->
+  expty:NCic.term NCicRefiner.expected_type ->
   mk_implicit: (bool -> 'alias) ->
   description_of_alias:('alias -> string) ->
   fix_instance:(DisambiguateTypes.domain_item -> 'alias list -> 'alias list) ->
   mk_choice:('alias -> NCic.term DisambiguateTypes.codomain_item) ->
   aliases:'alias DisambiguateTypes.Environment.t ->
   universe:'alias list DisambiguateTypes.Environment.t option ->
-  rdb:#NRstatus.status ->
   lookup_in_library:(
     DisambiguateTypes.interactive_user_uri_choice_type ->
     DisambiguateTypes.input_or_locate_uri_type ->
     DisambiguateTypes.Environment.key ->
     'alias list) ->
-  CicNotationPt.term Disambiguate.disambiguator_input ->
+  NotationPt.term Disambiguate.disambiguator_input ->
   ((DisambiguateTypes.domain_item * 'alias) list *
    NCic.metasenv *                  
    NCic.substitution *
@@ -38,23 +38,22 @@ val disambiguate_term :
   bool
   
 val disambiguate_obj :
+  #NCicCoercion.status ->
   mk_implicit:(bool -> 'alias) ->
   description_of_alias:('alias -> string) ->
   fix_instance:(DisambiguateTypes.domain_item -> 'alias list -> 'alias list) ->
   mk_choice:('alias -> NCic.term DisambiguateTypes.codomain_item) ->
   aliases:'alias DisambiguateTypes.Environment.t ->
   universe:'alias list DisambiguateTypes.Environment.t option ->
-  rdb:#NRstatus.status ->
   lookup_in_library:(
     DisambiguateTypes.interactive_user_uri_choice_type ->
     DisambiguateTypes.input_or_locate_uri_type ->
     DisambiguateTypes.Environment.key ->
      'alias list) ->
   uri:NUri.uri ->
-  string * int * CicNotationPt.term CicNotationPt.obj ->
+  string * int * NotationPt.term NotationPt.obj ->
   ((DisambiguateTypes.Environment.key * 'alias) list * NCic.metasenv *
    NCic.substitution * NCic.obj)
   list * bool
 
-val disambiguate_path: CicNotationPt.term -> NCic.term
-
+val disambiguate_path: #NCic.status -> NotationPt.term -> NCic.term
