@@ -9,7 +9,7 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nReference.ml 9543 2009-02-11 16:27:33Z tassi $ *)
+(* $Id: nReference.ml 12051 2012-05-17 11:32:52Z sacerdot $ *)
 
 exception IllFormedReference of string Lazy.t
 
@@ -25,10 +25,19 @@ type reference = Ref of NUri.uri * spec
 
 let eq = (==);;
 
+let compare (Ref (u1,s1)) (Ref (u2,s2)) =
+ let res = NUri.compare u1 u2 in
+  if res = 0 then compare s1 s2 else res
+;;
+
+let hash (Ref (uri,spec)) =
+ Hashtbl.hash spec + NUri.hash uri
+;;
+
 module OrderedStrings =
  struct
   type t = string
-  let compare (s1 : t) (s2 : t) = compare s1 s2
+  let compare (s1 : t) (s2 : t) = Pervasives.compare s1 s2
  end
 ;;
 
