@@ -9,7 +9,7 @@
      \ /   This software is distributed as is, NO WARRANTY.              
       V_______________________________________________________________ *)
 
-(* $Id: nUri.ml 9882 2009-06-15 16:22:18Z tassi $ *)
+(* $Id: nUri.ml 12524 2013-02-24 16:20:34Z fguidi $ *)
 
 type uri = int * string (* shareno, URI *)
 
@@ -20,6 +20,10 @@ let name_of_uri (_, uri) =
   Filename.chop_extension name
 ;;
 
+let baseuri_of_uri (_,uri) =
+ Filename.dirname uri
+;;
+
 module OrderedStrings =
  struct
   type t = string
@@ -44,13 +48,15 @@ fun s ->
 
 let eq = (==);;
 let compare (n1,_) (n2,_) = n2 - n1;;
+let hash (n,_) = n;;
 
 module HT = struct
         type t = uri
         let equal = eq
         let compare = compare
-        let hash (n,_) = n;;
+        let hash = hash;;
 end;;
 
 module UriHash = Hashtbl.Make(HT);;
 module UriMap = Map.Make(HT);;
+module UriSet = Set.Make(HT);;
