@@ -9,15 +9,18 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nUri.mli 9882 2009-06-15 16:22:18Z tassi $ *)
+(* $Id: nUri.mli 12524 2013-02-24 16:20:34Z fguidi $ *)
 
 type uri
 
 val string_of_uri: uri -> string
 val name_of_uri: uri -> string
+val baseuri_of_uri: uri -> string
 val uri_of_string: string -> uri
 val eq: uri -> uri -> bool
 val compare: uri -> uri -> int
+val hash: uri -> int
 
 module UriHash: Hashtbl.S with type key = uri
 module UriMap: Map.S with type key = uri
+module UriSet: Set.S with type elt = uri
