@@ -9,22 +9,27 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nCicRefiner.mli 10400 2009-10-06 13:37:01Z tassi $ *)
+(* $Id: nCicRefiner.mli 12479 2013-02-01 13:47:25Z fguidi $ *)
 
 exception RefineFailure of (Stdpp.location * string) Lazy.t;;
 exception Uncertain of (Stdpp.location * string) Lazy.t;;
 exception AssertFailure of string Lazy.t;;
 
+type 'a expected_type = [ `XTNone       (* unknown *)
+                        | `XTSome of 'a (* the given term *) 
+		        | `XTSort       (* any sort *)
+		        | `XTInd        (* any (co)inductive type *)
+                        ]
 val typeof :
- #NRstatus.status ->
+ #NCicCoercion.status ->
  ?localise:(NCic.term -> Stdpp.location) ->
   NCic.metasenv -> NCic.substitution -> NCic.context -> 
-  NCic.term -> NCic.term option -> (* term, expected type *)
+  NCic.term -> NCic.term expected_type -> (* term, expected type *)
     NCic.metasenv * NCic.substitution * NCic.term * NCic.term
     (*  menv, subst,refined term, type *)
 
 val typeof_obj :
- #NRstatus.status ->
+ #NCicCoercion.status ->
  ?localise:(NCic.term -> Stdpp.location) ->
   NCic.obj -> NCic.obj
 
