@@ -9,14 +9,14 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nCicUnification.mli 10569 2009-11-19 10:26:27Z ricciott $ *)
+(* $Id: nCicUnification.mli 12479 2013-02-01 13:47:25Z fguidi $ *)
 
 exception UnificationFailure of string Lazy.t;;
 exception Uncertain of string Lazy.t;;
 exception AssertFailure of string Lazy.t;;
 
 val unify :
-  #NRstatus.status ->
+  #NCicCoercion.status ->
   ?test_eq_only:bool -> (* default: false *)
   ?swap:bool -> (* default: false *)
   NCic.metasenv -> NCic.substitution -> NCic.context -> 
@@ -25,24 +25,29 @@ val unify :
 
 (* this should be moved elsewhere *)
 val fix_sorts: 
-  NCic.metasenv -> NCic.substitution -> 
+  #NCic.status -> NCic.metasenv -> NCic.substitution -> 
     NCic.term -> NCic.metasenv * NCic.term
 
+(* this should be moved elsewhere *)
+(* The term must be in whd *)
+val could_reduce: #NCicCoercion.status -> subst:NCic.substitution -> NCic.context -> NCic.term -> bool
+
 (* delift_type_wrt_terms st m s c t args
  *   lift t (length args) 
  *      [ rel 1 ... rel (len args) / lift (length args) (arg_1 .. arg_n) ]
  *)      
 val delift_type_wrt_terms:
-  #NRstatus.status -> 
+  #NCicCoercion.status -> 
   NCic.metasenv -> NCic.substitution -> NCic.context -> 
   NCic.term -> NCic.term list -> 
    NCic.metasenv * NCic.substitution * NCic.term
 
-val sortfy :
-    exn ->
-    NCic.metasenv ->
-    NCic.substitution ->
-    NCic.context ->
-    NCic.term -> NCic.metasenv * NCic.substitution * NCic.term
+val sortfy :#
+ NCic.status -> exn -> NCic.metasenv -> NCic.substitution -> NCic.context ->
+  NCic.term -> NCic.metasenv * NCic.substitution * NCic.term
+
+val indfy :#
+ NCic.status -> exn -> NCic.metasenv -> NCic.substitution -> NCic.context ->
+  NCic.term -> NCic.metasenv * NCic.substitution * NCic.term
 
 val debug : bool ref
