@@ -24,9 +24,10 @@
  *)
 
 module NCicIndexable : Discrimination_tree.Indexable
-with type input = NCic.term and type constant_name = NUri.uri
+with type input = NCic.term and type constant_name = NReference.reference
 
 module TermSet : Set.S with type elt = NCic.term
+module TermListSet : Set.S with type elt = NCic.term list
 
 module DiscriminationTree : Discrimination_tree.DiscriminationTree 
 with type constant_name = NCicIndexable.constant_name 
