@@ -11,4 +11,11 @@
 
 (* $Id: nCic.ml 9058 2008-10-13 17:42:30Z tassi $ *)
 
-val destruct_tac : 's NTacStatus.tactic
+val destruct_tac : string list option -> string list -> 's NTacStatus.tactic
+
+val mk_discriminator: (use_jmeq: bool) -> ?force:bool ->
+  string -> NCic.inductiveType -> int ->  
+  (#NTacStatus.tac_status as 's) -> string -> 's * NCic.obj
+
+exception ConstructorTooBig of string
+
