@@ -23,35 +23,36 @@
  * http://helm.cs.unibo.it/
  *)
 
-val eval_ast :
-  ?do_heavy_checks:bool ->
-  GrafiteTypes.status ->
-  string * int *
-  ((CicNotationPt.term, CicNotationPt.term,
-   CicNotationPt.term GrafiteAst.reduction, CicNotationPt.term CicNotationPt.obj, string)
-   GrafiteAst.statement) ->
-  (GrafiteTypes.status *
-   (DisambiguateTypes.domain_item * LexiconAst.alias_spec) option) list
+exception TryingToAdd of string Lazy.t
+exception EnrichedWithStatus of exn * GrafiteTypes.status
+exception AlreadyLoaded of string Lazy.t
+exception FailureCompiling of string * exn
+exception CircularDependency of string
 
+class status:
+ string ->
+  object
+   inherit GrafiteTypes.status
+   inherit ApplyTransformation.status
+  end
+
+val get_ast:
+ GrafiteTypes.status -> include_paths:string list -> GrafiteParser.parsable ->
+  GrafiteAst.statement
 
 (* heavy checks slow down the compilation process but give you some interesting
  * infos like if the theorem is a duplicate *)
-
-exception EnrichedWithStatus of exn * GrafiteTypes.status
-
-(* should be used only by the compiler since it looses the
-   * disambiguation_context (text,prefix_len,_) *)
-val eval_from_stream :
-  first_statement_only:bool ->
-  include_paths:string list ->
+val eval_ast :
+  include_paths: string list ->
   ?do_heavy_checks:bool ->
-  ?enforce_no_new_aliases:bool -> (* default true *)
-  ?watch_statuses:(GrafiteTypes.status -> unit) ->
   GrafiteTypes.status ->
-  Ulexing.lexbuf ->
-  (GrafiteTypes.status ->
-   (CicNotationPt.term, CicNotationPt.term,
-    CicNotationPt.term GrafiteAst.reduction, CicNotationPt.term CicNotationPt.obj, string)
-   GrafiteAst.statement -> unit) ->
+  string * int *
+  GrafiteAst.statement ->
   (GrafiteTypes.status *
-   (DisambiguateTypes.domain_item * LexiconAst.alias_spec) option) list
+   (DisambiguateTypes.domain_item * GrafiteAst.alias_spec) option) list
+
+val assert_ng: include_paths:string list -> string -> bool
+
+(* given a path to a ma file inside the include_paths, returns the
+   new include_paths associated to that file *)
+val read_include_paths: include_paths:string list -> string -> string list
