@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitaInit.ml 10434 2009-10-12 14:27:36Z tassi $ *)
+(* $Id: matitaInit.ml 12494 2013-02-04 17:38:31Z sacerdot $ *)
 
 type thingsToInitialize = 
   ConfigurationFile | Db | Environment | Getter | CmdLine | Registry
@@ -97,9 +97,6 @@ let initialize_db init_status =
   wants [ ConfigurationFile; CmdLine ] init_status;
   if not (already_configured [ Db ] init_status) then
     begin
-      if not (Helm_registry.get_bool "matita.system") then
-        MetadataTypes.ownerize_tables (Helm_registry.get "matita.owner");
-      LibraryDb.create_owner_environment ();
       Db::init_status
     end
   else
@@ -112,11 +109,6 @@ let initialize_environment init_status =
       Http_getter.init ();
       if Helm_registry.get_bool "matita.system" then
         Http_getter_storage.activate_system_mode ();
-      CicEnvironment.set_trust (* environment trust *)
-        (let trust =
-          Helm_registry.get_opt_default Helm_registry.get_bool
-            ~default:true "matita.environment_trust" in
-         fun _ -> trust);
       Getter::Environment::init_status
     end
   else
@@ -175,10 +167,9 @@ let parse_cmdline init_status =
     wants [Registry] init_status;
     let includes = ref [] in
     let default_includes = [ 
-      BuildTimeConf.stdlib_dir_devel;
-      BuildTimeConf.stdlib_dir_installed ; 
       BuildTimeConf.new_stdlib_dir_devel;
-      BuildTimeConf.new_stdlib_dir_installed ; 
+    (* CSC: no installed standard library!
+      BuildTimeConf.new_stdlib_dir_installed ; *)
     ] 
     in
     let absolutize s =
@@ -208,6 +199,9 @@ let parse_cmdline init_status =
           (Printf.sprintf ("<filename> Read configuration from filename"
              ^^ "\n    Default: %s")
             BuildTimeConf.matita_conf);
+        "-extract_ocaml", 
+          Arg.Unit (fun () -> Helm_registry.set_bool "extract_ocaml" true),
+          "Extract ocaml code";
         "-force",
             Arg.Unit (fun () -> Helm_registry.set_bool "matita.force" true),
             ("Force actions that would not be executed per default");
@@ -291,10 +285,3 @@ let parse_cmdline_and_configuration_file () =
 
 let initialize_environment () =
   status := initialize_environment !status
-
-let _ =
-  CicFix.init ();
-  Inversion_principle.init ();
-  CicRecord.init ();
-  CicElim.init ()
-;;
