@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitaTypes.ml 10354 2009-09-29 16:53:28Z tassi $ *)
+(* $Id: matitaTypes.ml 11131 2010-12-19 20:28:10Z sacerdot $ *)
 
 open Printf
 open GrafiteTypes
@@ -45,15 +45,9 @@ type abouts =
 type mathViewer_entry =
   [ `About of abouts  (* current proof *)
   | `Check of string  (* term *)
-  | `Cic of Cic.term * Cic.metasenv
   | `NCic of NCic.term * NCic.context * NCic.metasenv * NCic.substitution
   | `Dir of string  (* "directory" in cic uris namespace *)
-  | `HBugs of [ `Tutors ] (* list of available HBugs tutors *)
-  | `Metadata of [ `Deps of [`Fwd | `Back] * UriManager.uri ]
-  | `Uri of UriManager.uri (* cic object uri *)
   | `NRef of NReference.reference (* cic object uri *)
-  | `Whelp of string * UriManager.uri list (* query and results *)
-  | `Univs of UriManager.uri
   ]
 
 let string_of_entry = function
@@ -66,24 +60,9 @@ let string_of_entry = function
   | `About `Grammar -> "about:grammar"
   | `About `Hints -> "about:hints"
   | `Check _ -> "check:"
-  | `Cic (_, _) -> "term:"
   | `NCic (_, _, _, _) -> "nterm:"
   | `Dir uri -> uri
-  | `HBugs `Tutors -> "hbugs:/tutors/"
-  | `Metadata meta ->
-      "metadata:/" ^
-      (match meta with
-      | `Deps (dir, uri) ->
-          "deps/" ^
-          let suri =
-            let suri = UriManager.string_of_uri uri in
-            let len = String.length suri in
-            String.sub suri 4 (len - 4) in (* strip "cic:" prefix *)
-          (match dir with | `Fwd -> "forward" | `Back -> "backward") ^ suri)
-  | `Uri uri -> UriManager.string_of_uri uri
   | `NRef nref -> NReference.string_of_reference nref
-  | `Whelp (query, _) -> query
-  | `Univs uri -> "univs:" ^ UriManager.string_of_uri uri
 
 let entry_of_string = function
   | "about:blank" -> `About `Blank
@@ -96,16 +75,3 @@ let entry_of_string = function
   | "about:grammar"    -> `About `Grammar
   | _ ->  (* only about entries supported ATM *)
       raise (Invalid_argument "entry_of_string")
-
-class type mathViewer =
-  object
-    (** @param reuse if set reused last opened cic browser otherwise 
-     *  opens a new one. default is false
-     *)
-    method show_entry: ?reuse:bool -> mathViewer_entry -> unit
-    method show_uri_list:
-      ?reuse:bool -> entry:mathViewer_entry -> UriManager.uri list -> unit
-    method screenshot: 
-      GrafiteTypes.status -> NCic.metasenv -> NCic.metasenv ->
-        NCic.substitution -> string -> unit
-  end
